/*
   Original C version: Kendall Bennett 10-1-97  
   MFCified and jpeg support: Dan Edwards	10-27-97
*/

#include "stdafx.h"
#include "ShowBmp.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_PAINT()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_OPTIONSSTRETCH, OnOptionsstretch)
	ON_UPDATE_COMMAND_UI(ID_OPTIONSSTRETCH, OnUpdateOptionsstretch)
	ON_WM_ACTIVATEAPP()
	ON_WM_QUERYNEWPALETTE()
	ON_WM_SIZE()
	ON_WM_PALETTECHANGED()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_OPTIONS_INCLUDESTATIC, OnOptionsIncludestatic)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_INCLUDESTATIC, OnUpdateOptionsIncludestatic)
	ON_COMMAND(ID_OPTIONS_ANIMATEPALETTE, OnOptionsAnimatepalette)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_ANIMATEPALETTE, OnUpdateOptionsAnimatepalette)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	winDC=NULL;
	memDC=NULL;
	m_bStretch=FALSE;
	includeStatic = FALSE;
	animatePalette = FALSE;
}

CMainFrame::~CMainFrame()
{
	MGL_exit();

}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::InitMGL(void)
/****************************************************************************
*
* Function:     InitMGL
*
* Description:  Initialise the MGL and load all bitmap resources required by
*               the application. This is where the application should allocate
*               space for all MGL memory DC's used for storing sprites etc
*               and load the sprites from disk.
*
****************************************************************************/
{
    /* Register all the MGL packed pixel drivers, since we need them all for
     * loading and displaying bitmaps of different pixel depths.
     */
    MGL_registerDriver(MGL_PACKED1NAME,PACKED1_driver);
    MGL_registerDriver(MGL_PACKED4NAME,PACKED4_driver);
    MGL_registerDriver(MGL_PACKED8NAME,PACKED8_driver);
    MGL_registerDriver(MGL_PACKED16NAME,PACKED16_driver);
    MGL_registerDriver(MGL_PACKED24NAME,PACKED24_driver);
    MGL_registerDriver(MGL_PACKED32NAME,PACKED32_driver);

    /* Initialise the MGL for Windowed output only */
    MGL_initWindowed("..\\..\\..\\");
}


MGLDC *CMainFrame::LoadJPEGIntoMemDC(MGLDC *dc, const char *jpegName)
/****************************************************************************
*
* Function:     LoadBitmapIntoMemDC
* Parameters:   dc          - Display dc
*               bitmapName  - Name of bitmap file to load
* Returns:      Pointer to valid memory DC with bitmap loaded
*
* Description:  Loads the specified bitmap into a memory DC with the
*               same dimensions as the bitmap on disk, but with the same
*               pixel depth and format used by the display DC (for maximum
*               blt performance). The MGL automatically handles pixel
*               format conversion for us when we load the bitmap into our
*               memory DC, and the bitmap's palette is loaded into the
*               memory DC's palette.
*
****************************************************************************/
{
    int             width,height,bits,bitmapBits;
    pixel_format_t  pf;
    MGLDC           *memDC;

    /* Get dimensions of bitmap image on disk */
    if (!MGL_getJPEGSize(jpegName,&width,&height,&bitmapBits,&pf))
        MGL_fatalError("Cant locate jpeg file!");

    /* Get bitmap depth and pixel format for memory DC */ 
    GetMemoryBitmapDepth(dc,&bits,&pf);

    /* Create the memory DC and load the bitmap file into it */
    if ((memDC = MGL_createMemoryDC(width,height,bits,&pf)) == NULL)
        MGL_fatalError("Not enough memory to load jpeg!");
    if (!MGL_loadJPEGIntoDC(memDC,jpegName,0,0,true))
        MGL_fatalError("Cant locate jpeg file!");

    /* Load bitmap palette into Window DC and realize it */
    LoadBitmapPalette(winDC,memDC);
    return memDC;
}

MGLDC *CMainFrame::LoadBitmapIntoMemDC(MGLDC *dc, const char *bitmapName)
/****************************************************************************
*
* Function:     LoadBitmapIntoMemDC
* Parameters:   dc          - Display dc
*               bitmapName  - Name of bitmap file to load
* Returns:      Pointer to valid memory DC with bitmap loaded
*
* Description:  Loads the specified bitmap into a memory DC with the
*               same dimensions as the bitmap on disk, but with the same
*               pixel depth and format used by the display DC (for maximum
*               blt performance). The MGL automatically handles pixel
*               format conversion for us when we load the bitmap into our
*               memory DC, and the bitmap's palette is loaded into the
*               memory DC's palette.
*
****************************************************************************/
{
    int             width,height,bits,bitmapBits;
    pixel_format_t  pf;
    MGLDC           *memDC;

    /* Get dimensions of bitmap image on disk */
    if (!MGL_getBitmapSize(bitmapName,&width,&height,&bitmapBits,&pf))
        MGL_fatalError("Cant locate bitmap file!");

    /* Get bitmap depth and pixel format for memory DC */ 
    GetMemoryBitmapDepth(dc,&bits,&pf);

    /* Create the memory DC and load the bitmap file into it */
    if ((memDC = MGL_createMemoryDC(width,height,bits,&pf)) == NULL)
        MGL_fatalError("Not enough memory to load bitmap!");
    if (!MGL_loadBitmapIntoDC(memDC,bitmapName,0,0,true))
        MGL_fatalError("Cant locate bitmap file!");

    /* Load bitmap palette into Window DC and realize it */
    LoadBitmapPalette(winDC,memDC);
    return memDC;
}

void CMainFrame::LoadBitmapPalette(MGLDC *winDC,MGLDC *memDC)
/****************************************************************************
*
* Function:     LoadBitmapPalette
* Parameters:   winDC   - Window DC to load palette into
*               memDC   - Memory DC to get palette values from
* Returns:      Pointer to valid memory DC with bitmap loaded
*
* Description:  Load the palette from the memory DC bitmap into the Window DC
*               and realize it to make it the hardware palette. Note that
*               we also realize the palette for the loaded bitmap in the
*               memory DC to re-build the DIB's color table when we switch
*               to and from SYSPAL_NOSTATIC mode.
*
****************************************************************************/
{
    palette_t       pal[256];

    MGL_getPalette(memDC,pal,MGL_getPaletteSize(memDC),0);
    MGL_setPalette(winDC,pal,MGL_getPaletteSize(winDC),0);
    MGL_realizePalette(memDC,MGL_getPaletteSize(memDC),0,false);
    MGL_realizePalette(winDC,MGL_getPaletteSize(winDC),0,false);
}

void CMainFrame::GetMemoryBitmapDepth(MGLDC *dc,int *bits,pixel_format_t *pf)
/****************************************************************************
*
* Function:     GetMemoryBitmapDepth
* Parameters:   dc      - Window DC
*               bits    - Place to store pixel depth
*               pf      - Place to store pixel format
* Returns:      Pointer to valid memory DC with bitmap loaded
*
* Description:  Under Windows 3.1, the only bitmaps we can blt to the display
*               are 8 bit bitmaps. Under Windows NT and '95 we can blt
*               bitmaps of any color depth, so we create our memory DC with
*               the same depth and format as the current Windows display mode.
*
****************************************************************************/
{
	if (!haveTrueColor)
        *bits = 8;
    else {
        *bits = MGL_getBitsPerPixel(dc);
        MGL_getPixelFormat(dc,pf);
        }
}

void CMainFrame::CreateMGLDeviceContexts(HWND hwnd)
/****************************************************************************
*
* Function:     CreateMGLDeviceContexts
* Parameters:   hwnd    - Handle to Window used for output
*
* Description:  Sets up the MGL for output to the client region of the
*               specified window. This function will destroy any previously
*               existing MGL device contexts, and will re-build all the
*               DC's for the Window. Hence this routine is always called
*               from the Windows WM_SIZE message handler, which will always
*               be called before the first WM_PAINT message is received,
*               allowing the MGL device contexts to be created with the
*               appropriate size information.
*
****************************************************************************/
{
    /* Destroy the previous Window DC if it existed */
    if (winDC)
        MGL_destroyDC(winDC);

    /* Create the main MGL Windowed DC for interfacing our main window */
    if ((winDC = MGL_createWindowedDC(hwnd)) == NULL)
        MGL_fatalError("Unable to create Windowed DC!");		  

    /* Re-load the bitmap palette */
    if (memDC)
        LoadBitmapPalette(winDC,memDC);
}

void CMainFrame::AppActivate(BOOL fActive)
/****************************************************************************
*
* Function:     AppActivate
* Parameters:   fActive - True if app is activating
*
* Description:  If the application is activating, then swap the system
*               into SYSPAL_NOSTATIC mode so that our palettes will display
*               correctly.
*
****************************************************************************/
{
	HDC hdc = ::GetDC(NULL);
    int i;

    if (fActive && (GetSystemPaletteUse(hdc) == SYSPAL_STATIC)) {
        /* Switch to SYSPAL_NOSTATIC */
		SetSystemPaletteUse(hdc, SYSPAL_NOSTATIC);
		}
	else if (!fActive) {
		/* Always switch back to SYSPAL_STATIC */
		SetSystemPaletteUse(hdc, SYSPAL_STATIC);
        }
	::ReleaseDC(NULL,hdc);
}

void CMainFrame::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	static BOOL bFirst=TRUE;

	if(bFirst)
	{					
		bFirst=FALSE;
		haveTrueColor = TRUE;

		InitMGL();
		CreateMGLDeviceContexts(GetSafeHwnd());
		//memDC = LoadBitmapIntoMemDC(winDC, "scrlbrd.bmp");
	}


	if(memDC) 
	{

		if(m_bStretch) 
		{
			MGL_stretchBltCoord(winDC,memDC,
								0,0,MGL_sizex(memDC)+1,MGL_sizey(memDC)+1,
								0,0,MGL_sizex(winDC)+1,MGL_sizey(winDC)+1);
		}
		else
		{
			MGL_bitBltCoord(winDC,memDC,
								0,0,MGL_sizex(memDC)+1,MGL_sizey(memDC)+1,
								0,0,MGL_REPLACE_MODE);
		}
	}

	// Do not call CFrameWnd::OnPaint() for painting messages
}

void CMainFrame::OnFileOpen() 
{
	CFileDialog Dlg(TRUE,    // open
                   NULL,    // no default extension
                   NULL,    // no initial file name
                   OFN_FILEMUSTEXIST | OFN_HIDEREADONLY,
						"Image files (*.JPG, *.BMP)|*.JPG;*.BMP|All files (*.*)|*.*||");

	if(Dlg.DoModal()==IDOK)
	{		
		if (memDC)
			MGL_destroyDC(memDC); 

		CString strExt=Dlg.GetFileExt();
		strExt.MakeLower();

		if(strExt=="jpg")
			memDC = LoadJPEGIntoMemDC(winDC, Dlg.GetPathName());
		else
			memDC = LoadBitmapIntoMemDC(winDC, Dlg.GetPathName());

		Invalidate(TRUE);
	}
}

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask) 
{
	CFrameWnd::OnActivateApp(bActive, hTask);

	fAppActive = bActive;
   if (!includeStatic)
       AppActivate(fAppActive);
	
	MGL_appActivate(winDC,fAppActive);
	
}

BOOL CMainFrame::OnQueryNewPalette() 
{
	if (winDC && MGL_activatePalette(winDC,false)) 
	{
		Invalidate(TRUE);
		return TRUE;
	}
	return FALSE;

	//return CFrameWnd::OnQueryNewPalette();
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd) 
{
	CFrameWnd::OnPaletteChanged(pFocusWnd);
	
	OnQueryNewPalette();	
}

void CMainFrame::OnSize(UINT nType, int cx, int cy) 
{
	CFrameWnd::OnSize(nType, cx, cy);
	
	CreateMGLDeviceContexts(GetSafeHwnd());
}



BOOL CMainFrame::OnEraseBkgnd(CDC* pDC) 
{
	// if there is a bitmap loaded and in stretch mode don't let windows draw background, prevents horendous flicker
	if(memDC && m_bStretch)
		return TRUE;
	else
		return CFrameWnd::OnEraseBkgnd(pDC);
}

void CMainFrame::OnOptionsstretch() 
{
	m_bStretch=!m_bStretch;
	Invalidate(TRUE);
}

void CMainFrame::OnUpdateOptionsstretch(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bStretch);	
}

void CMainFrame::OnOptionsIncludestatic() 
{
	includeStatic=!includeStatic;	
	
	if (includeStatic) 
		AppActivate(true);
	else
		AppActivate(false);
		

	

	/* Rebuild the MGL device context to re-build the
	* palette for the loaded bitmap.
	*/
	CreateMGLDeviceContexts(GetSafeHwnd());
	InvalidateRect(FALSE);

}

void CMainFrame::OnUpdateOptionsIncludestatic(CCmdUI* pCmdUI) 
{	
	if (MGL_getBitsPerPixel(winDC) == 8)
		pCmdUI->SetCheck(includeStatic );
	else
		pCmdUI->Enable(FALSE);
}

void CMainFrame::OnOptionsAnimatepalette() 
{
	animatePalette=!animatePalette;
	if(animatePalette)
		SetTimer(PALETTETIMER, 100, NULL);
	else
		KillTimer(PALETTETIMER);
}					

void CMainFrame::OnUpdateOptionsAnimatepalette(CCmdUI* pCmdUI) 
{	
	if (MGL_getBitsPerPixel(winDC) == 8)
		pCmdUI->SetCheck(animatePalette);
	else
		pCmdUI->Enable(FALSE);
}

void CMainFrame::OnTimer(UINT nIDEvent) 
{

	if(nIDEvent==PALETTETIMER)
	{
		int	palsize;

		/* Rotate the hardware palette */
		palsize = MGL_getPaletteSize(winDC);
		MGL_rotatePalette(winDC,palsize-2,1,MGL_ROTATE_UP);
		MGL_realizePalette(winDC,palsize,0,true);
	}

	CFrameWnd::OnTimer(nIDEvent);
}
